package com.bonait.bnframework.modules.home.activity;

import android.os.Bundle;
import androidx.annotation.NonNull;

import com.bonait.bnframework.business.ConfigData;
import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.constant.DataBus;
import com.bonait.bnframework.common.helper.I.IThread;
import com.bonait.bnframework.common.helper.MessageLog;
import com.bonait.bnframework.common.helper.ThreadManager;
import com.bonait.bnframework.common.modbus.ModbusTcpServer;
import com.bonait.bnframework.common.utils.NetworkUtils;
import com.bonait.bnframework.modules.home.fragment.DingDanfragment;
import com.bonait.bnframework.modules.home.fragment.GongnengFragment;
import com.bonait.bnframework.modules.home.fragment.JiaoYanFragment;
import com.google.android.material.bottomnavigation.BottomNavigationView;

import androidx.fragment.app.Fragment;
import androidx.viewpager.widget.ViewPager;

import android.util.Log;
import android.view.KeyEvent;
import android.view.MenuItem;

import com.bonait.bnframework.R;
import com.bonait.bnframework.common.constant.MessageName;
import com.bonait.bnframework.common.message.MessageLooper;
import com.bonait.bnframework.common.message.MessageManager;
import com.bonait.bnframework.manager.ActivityLifecycleManager;
import com.bonait.bnframework.common.base.BaseActivity;
import com.bonait.bnframework.common.utils.ToastUtils;
import com.bonait.bnframework.modules.home.fragment.Home1Fragment;
import com.bonait.bnframework.modules.home.fragment.Home2Fragment;
import com.bonait.bnframework.modules.home.adapter.FragmentAdapter;
import com.bonait.bnframework.modules.mine.fragment.MyFragment;
import com.lzy.okgo.OkGo;
import com.qmuiteam.qmui.widget.QMUIViewPager;

import java.util.ArrayList;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

public class BottomNavigation2Activity extends BaseActivity {


    @BindView(R.id.navigation)
    BottomNavigationView bottomNavigationView;
    @BindView(R.id.viewpager)
    QMUIViewPager viewPager;

    private MenuItem menuItem;
    private long exitTime = 0;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_bottom_navigation2);
        ButterKnife.bind(this);

        initFragment();
        viewPager.addOnPageChangeListener(pageChangeListener);
        // 设置viewPager缓存多少个fragment
        viewPager.setOffscreenPageLimit(3);
        bottomNavigationView.setOnNavigationItemSelectedListener(mOnNavigationItemSelectedListener);
        Init();
    }

    @Override
    protected void onDestroy() {
        ConfigData.getInstance().ColsePLC();
        //AliyunIOTManager.getInstance().CloseDev();
        super.onDestroy();
    }

    /**
     * viewPager里添加fragment
     */
    private void initFragment() {
        List<Fragment> fragmentList= new ArrayList<>();
        fragmentList.add(new Home2Fragment());
        fragmentList.add(new Home1Fragment());
        fragmentList.add(new MyFragment());
        FragmentAdapter fragmentAdapter = new FragmentAdapter(getSupportFragmentManager(),fragmentList);
//        fragmentAdapter.addFragment(new Home2Fragment());
//        fragmentAdapter.addFragment(new Home1Fragment());
//        fragmentAdapter.addFragment(new MyFragment());
        viewPager.setAdapter(fragmentAdapter);

        MessageManager.getInstance().registerMessageReceiver(this, MessageName.SelectZY, new MessageLooper.OnMessageListener() {
            @Override
            public void onMessage(Object msg) {
                if (msg != null) {
                    viewPager.setCurrentItem(1);
                }
            }
        });
    }

    //-------------------------配置viewPager与fragment关联----------------------------//

    /**
     * 配置bottom底部菜单栏监听器，手指点击底部菜单监听
     */
    private BottomNavigationView.OnNavigationItemSelectedListener mOnNavigationItemSelectedListener
            = new BottomNavigationView.OnNavigationItemSelectedListener() {

        @Override
        public boolean onNavigationItemSelected(@NonNull MenuItem item) {
            switch (item.getItemId()) {
                case R.id.bottom_navigation_1:
                    viewPager.setCurrentItem(0);
                    return true;
                case R.id.bottom_navigation_2:
                    viewPager.setCurrentItem(1);
                    return true;
                case R.id.bottom_navigation_3:
                    viewPager.setCurrentItem(2);
                    return true;
            }
            return false;
        }
    };


    /**
     * 配置ViewPager监听器，手指滑动监听
     */
    private ViewPager.OnPageChangeListener pageChangeListener = new ViewPager.OnPageChangeListener() {
        @Override
        public void onPageScrolled(int position, float positionOffset, int positionOffsetPixels) {
            MenuItem menuItem = bottomNavigationView.getMenu().getItem(position);
        }

        @Override
        public void onPageSelected(int position) {
            menuItem = bottomNavigationView.getMenu().getItem(position);
            menuItem.setChecked(true);
        }

        @Override
        public void onPageScrollStateChanged(int state) {

        }
    };

    @Override
    protected boolean canDragBack() {
        return viewPager.getCurrentItem() == 0;
    }

    /**
     * 重写返回键，实现双击退出程序效果
     */
    @Override
    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            if (System.currentTimeMillis() - exitTime > 2000) {
                ToastUtils.normal("再按一次退出程序");
                exitTime = System.currentTimeMillis();
            } else {
                OkGo.getInstance().cancelAll();
                overridePendingTransition(android.R.anim.fade_in, android.R.anim.fade_out);
                ActivityLifecycleManager.get().appExit();
            }
            return true;
        }
        return super.onKeyDown(keyCode, event);
    }

    /**
     * 初始化
     */
    public void  Init()
    {
        //1.同步时间
        NetworkUtils.SynchronizationTime();
        //主持
        ConfigData.getInstance().GetOrganize(this);
        //判断连接环境
        ConfigData.getInstance().ToggleEnvironment();
        //2.初始化PLC
        ReconnectModbus();
        //初始化阿里云连接
        //AliyunIOTManager.getInstance().OpenDev(this);
    }

    /**
     * 重新连接plc
     */
    public void ReconnectModbus()
    {
        try {

            ThreadManager.Get().StartLong("PLC断线重连线程", true, new IThread() {
                @Override
                public void Run() throws InterruptedException {
                    try {
                        if(ConfigName.getInstance().PlcIsConnect)
                        {
                            //ping 不通
                            boolean status = ModbusTcpServer.ping2(ConfigName.getInstance().Address,1,1);
                            if(!status) //ping 不通 连接
                            {
                                MessageLog.ShowInfo("PLC状态断开,尝试连接...");
                                ConfigName.getInstance().PlcIsConnect=false;
                            }
                        }else
                        {
                            boolean status = ModbusTcpServer.ping2(ConfigName.getInstance().Address,1,1);
                            if(status)
                            {
                                MessageLog.ShowInfo("设备 " + ConfigName.getInstance().Address + " PLC通讯正常,准备连接！");
                                ModbusTcpServer.ConnectPLC();
                            }else
                            {
                                MessageLog.ShowInfo("PLC状态断开,尝试连接...");
                                ConfigName.getInstance().PlcIsConnect=false;
                            }
                        }
                        Thread.sleep(10000);
                    } catch (Exception e) {
                        Log.i("PLC", "PLC重连接失败!"+e.getMessage());
                    }
                }
                @Override
                public void RunComplete() throws InterruptedException {
                }
            });
        }catch (Exception e) {
            MessageLog.ShowInfo("重新连接Modbus异常，" +e.getMessage());
        }
    }
}
