package com.bonait.bnframework.modules.home.fragment;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.content.Context;
import android.os.Bundle;
import android.view.LayoutInflater;
import android.view.View;

import com.bonait.bnframework.R;
import com.bonait.bnframework.common.base.BaseFragment;
import com.bonait.bnframework.common.constant.MessageName;
import com.bonait.bnframework.common.db.QueryDB;
import com.bonait.bnframework.common.db.mode.BPA_GOODS;
import com.bonait.bnframework.common.db.mode.BPA_GOODSTYPE;
import com.bonait.bnframework.common.linktab.LinkMode;
import com.bonait.bnframework.common.linktab.TopItemDecoration;
import com.bonait.bnframework.common.linktab.good.GoodLeftAdapter;
import com.bonait.bnframework.common.linktab.good.GoodRightAdapter;
import com.bonait.bnframework.common.linktab.makegood.MakeGoodLeftAdapter;
import com.bonait.bnframework.common.linktab.makegood.MakeGoodRightAdapter;
import com.bonait.bnframework.common.message.MessageLooper;
import com.bonait.bnframework.common.message.MessageManager;
import com.bonait.bnframework.common.utils.ToastUtils;
import com.litao.slider.NiftySlider;
import com.orhanobut.logger.Logger;
import com.qmuiteam.qmui.widget.QMUITopBarLayout;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;

import butterknife.BindView;
import butterknife.ButterKnife;

public class MakeGoodFragment extends BaseFragment {

    @BindView(R.id.topbar)
    QMUITopBarLayout mTopBar;//顶部标题

    /**
     * 左边分类
     */
    @BindView(R.id.rv_left)
    RecyclerView rv_left;

    /**
     * 右边商品
     */
    @BindView(R.id.rv_right)
    RecyclerView rv_right;

    /**
     * 商品类型
     */
    ArrayList<BPA_GOODSTYPE> goodstypes=new ArrayList<>();

    private Context context;

    public MakeGoodFragment() {
    }

    @Override
    protected View onCreateView() {
        View root = LayoutInflater.from(getActivity()).inflate(R.layout.fragment_make_good, null);
        ButterKnife.bind(this, root);
        return root;
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        context = getContext();
        initTopBar();
        Initdata();
    }
    public List<LinkMode> Fdata = new ArrayList<>();
    /**
     * 初始化数据
     */
    public void Initdata()
    {
        //1.商品类型
        goodstypes = QueryDB.GetGoodsTypeALL();

        Fdata.clear();

        int i=0;
        for(BPA_GOODSTYPE item :goodstypes)
        {
            ArrayList<BPA_GOODS> goodsm= QueryDB.GetGoodsTypeId(item.id);
            Fdata.add(new LinkMode(item, Arrays.asList(goodsm.toArray()), i==0?true:false));
            i++;
        }

        LinearLayoutManager manager = (LinearLayoutManager) rv_right.getLayoutManager();
        MakeGoodLeftAdapter leftAdapter = new MakeGoodLeftAdapter(R.layout.item_left_makegood, Fdata,getResources(),manager);
        MakeGoodRightAdapter rightAdapter = new MakeGoodRightAdapter(R.layout.item_right_lc, Fdata,getResources());
        rv_left.setAdapter(leftAdapter);
        rv_right.setAdapter(rightAdapter);


        //添加分组悬浮效果
        List<String> typeListener = new ArrayList<>();
        for (LinkMode item : Fdata) {
            typeListener.add(((BPA_GOODSTYPE) item.type).name);
        }

        if(rv_right.getItemDecorationCount()>0)
        {
            rv_right.removeItemDecorationAt(0);
        }
        TopItemDecoration top = new TopItemDecoration(context, typeListener);
        rv_right.addItemDecoration(top);

        //左侧联动
        rv_right.addOnScrollListener(new RecyclerView.OnScrollListener() {
            @Override
            public void onScrolled(@NonNull RecyclerView recyclerView, int dx, int dy) {
                super.onScrolled(recyclerView, dx, dy);
                int firstItemPosition = manager.findFirstVisibleItemPosition();
                if (firstItemPosition != -1 && dy != 0) {
                    rv_left.smoothScrollToPosition(firstItemPosition);
                    leftAdapter.setChoose(firstItemPosition);
                }
            }
        });
    }
    /**
     * 初始化TopBar
     */
    private void initTopBar() {
        mTopBar.setTitle("智慧菠萝点餐系统");

        //打开
        MessageManager.getInstance().registerMessageReceiver(getActivity(), MessageName.ClickMakeGoodNew, new MessageLooper.OnMessageListener() {
            @Override
            public void onMessage(Object msg) {
                if (msg != null) {
                    BPA_GOODS goods=(BPA_GOODS) msg;
                    ToastUtils.info("商品:"+goods.name+",加入队列中...");
                }
            }
        });
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
        Logger.d("第一页销毁");
    }

    /**
     * 当在activity设置viewPager + BottomNavigation + fragment时，
     * 为防止viewPager左滑动切换界面，与fragment左滑返回上一界面冲突引起闪退问题，
     * 必须加上此方法，禁止fragment左滑返回上一界面。
     * <p>
     * 切记！切记！切记！否则会闪退！
     * <p>
     * 当在fragment设置viewPager + BottomNavigation + fragment时，则不会出现这个问题。
     */
    @Override
    protected boolean canDragBack() {
        return false;
    }
}